/* The program "branchdist" compares two trees and returns 
the number of common splits (Robinson -- Foulds distance) to the stdin */
#include "tree.h"

int main (int argc, char *argv[])
{
  unsigned int i;
  int number, n;
  int distance;
  unsigned int len = 128;
  FILE *inflow;
  struct tree intree1, intree2;
  char *newick;
  char c;

  /* Checking command line */
  if (argc < 2)
  {
    fprintf(stderr, "Usage: %s <input tree 1> [<input tree 2>]\n", argv[0]);
    fprintf(stderr, "Example: %s tree1.tre tree2.tre\n", argv[0]);
    exit (1);
  }

  inflow = fopen(argv[1], "r");
  if ( inflow == NULL ) {
    fprintf(stderr, "Can not open input file \"%s\"!\n", argv[1]);
    exit(1);
  }

  newick = (char*) malloc(sizeof(char)*len);
  i = 0;
  c = '\0';
  while (!feof(inflow) && (c != ';') ) {
    c = fgetc(inflow);
    if ( !feof(inflow) && c != '\n' && c != '\r') {
      i++;
      if (i > len - 2) {
        len = 2*len;
        newick = (char*)realloc(newick, sizeof(char)*len);
      }
      newick[i-1] = c;
    }
  }
  if ( feof(inflow) ) {
    fprintf(stderr, "Wrong tree format in \"%s\"!\n", argv[1]);
    exit(1);
  }
  newick[i] = '\0';
/*  fprintf(stderr, "First tree %s\n", newick); */
  intree1 = readbrackets(newick);
  free(newick);
  fprintf(stderr, "First tree:\t%s\n", writebrackets(intree1, 0));

  if (argc > 2) { /* second tree is in separate file */
    fclose(inflow);
    inflow = fopen(argv[2], "r");
    if ( inflow == NULL ) {
      fprintf(stderr, "Can not open input file \"%s\"!\n", argv[2]);
      exit(1);
    }
  }
  newick = (char*) malloc(sizeof(char)*len);
  i = 0;
  c = '\0';
  while (!feof(inflow) && (c != ';') ) {
    c = fgetc(inflow);
    if ( !feof(inflow) && c != '\n' && c != '\r') {
      i++;
      if (i > len - 2) {
        len = 2*len;
        newick = (char*)realloc(newick, sizeof(char)*len);
      }
      newick[i-1] = c;
    }
  }
  if ( feof(inflow) ) {
    if (argc > 2) {
      fprintf(stderr, "Wrong tree format in \"%s\"!\n", argv[2]);
    }
    else {
      fprintf(stderr, "Only one tree in \"%s\"!\n", argv[1]);
    }
    exit(1);
  }
  fclose(inflow);
  newick[i] = '\0';
/*  fprintf(stderr, "Second tree: %s\n", newick); */
  intree2 = readbrackets(newick);
  free(newick);
  fprintf(stderr, "Second tree:\t%s\n", writebrackets(intree2, 0));
  distance = branchdist(intree1, intree2);
  n = intree1.branchnum + intree2.branchnum - intree1.leavesnum - intree2.leavesnum;
  printf("%d\t%6.4f\n", distance, (float)distance/n );
  return 0;
} /* main */

