/* subtree takes a tree and a list of species' names
and create a tree induced by the input tree on those species */
#include "tree.h"

int main  (int argc, char *argv[])
{
  unsigned int i, m;
  unsigned int len = 128;
  FILE *inflow1;
  FILE *inflow2;
  FILE *outflow;
  struct tree intree, restree;
  char c;
  char line[128];
  char **list;
  char *newick;
  char *result;

  /* Checking command line */
  if (argc < 3)
  {
    fprintf(stderr, "Usage: %s <input tree> <list file> <output tree>\n", argv[0]);
    fprintf(stderr, "Example: %s mytree.tre mylist.txt subtree.tre\n", argv[0]);
    exit(1);
  }

  inflow1 = fopen(argv[1], "r");
  if ( inflow1 == NULL ) {
    perror("Can not open input tree file!\n");
    exit(1);
  }

  inflow2 = fopen(argv[2], "r");
  if ( inflow2 == NULL ) {
    perror("Can not open input list file!\n");
    exit(1);
  }

  newick = (char*) malloc(sizeof(char)*len);
  i = 0;
  while (!feof(inflow1) ) {
    c = fgetc(inflow1);
    if ( !feof(inflow1) && c != '\n' && c != '\r') {
      i++;
      if (i > len - 2) {
        len = 2*len;
        newick = (char*)realloc(newick, sizeof(char)*len);
      }
      newick[i-1] = c;
    }
  }
  fclose(inflow1);
  newick[i] = '\0';

  intree = readbrackets(newick);
  if ( intree.leavesnum < 2 ) {
    fprintf(stderr, "Wrong tree format in \"%s\"\n", argv[2]);
    exit(1);
  }
  fprintf(stderr, "Input tree is %s\n", writebrackets(intree,0));

  m = intree.leavesnum; /* probably list contains less elements */
  list = (char**)malloc(sizeof(char*)*m);
  
  i = 0;
  while (!feof(inflow2)) {
    fgets(line, 127, inflow2);
    if (!feof(inflow2)) {
      if ( line[strlen(line)-1] != '\n' ) {
        fprintf(stderr, "Too long string in listfile:\n%s...\n", line);
        exit(1);
      }
      list[i] = (char*)calloc(strlen(line), sizeof(char));
      sscanf(line,"%s",list[i]);
      i++;
      if ( i > m - 1 ) { 
        m = m + intree.leavesnum;   /* a huge list ... */
        list = (char**)realloc(list, sizeof(char*)*m);
      }
    } /* if !feof */
  }/* while */

  restree = subtree(intree, list, i);

  outflow = fopen(argv[3], "w");
  if ( outflow == NULL ) {
    perror("Can not open output file!\n");
    printf("%s\n", writebrackets(restree, 0));
  }
  else {
    fprintf(outflow, "%s\n", writebrackets(restree, 0));
    fclose(outflow);
  }
  return 0;

} /* main */