#ifndef _HEADER_H_
#define _HEADER_H_
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <ctype.h>

/* Structure for tree */
struct tree {
  unsigned int leavesnum; /* number of leaves */
  unsigned int branchnum; /* number of branches, including trivial ones */
  char **leaf; /* leaves'names */
  unsigned int **branch; /* branches as strings of 0  and 1 */
  unsigned int rooted; /* 1 - rooted, 0 - unrooted */
  unsigned int root; /* index of branch */
  unsigned int phylogram; /* 1 if lengths are really lengths, 0 otherwise */
  float *length; /* branch lengths */
  float rootlocation; /* distance from the root to the node from the side of leaf #1 (index 0) */
};

struct distancematrix {
  unsigned int speciesnum; /* number of species */
  char **species; /* species'names */
  float **distance; /* distances */
};

struct tree readbrackets(char *brackets); /* converting string with Newick to rooted tree */

char *writebrackets(struct tree intree, int writelengths); /* converting tree to Newick */

struct tree midpoint(struct tree intree); /* rooting to midpoint */

float treedistance(struct tree intree, unsigned int leaf1, unsigned int leaf2); /* distance between two leaves */

unsigned int combdistance(struct tree intree, unsigned int leaf1, unsigned int leaf2); 
/* combinatorial distance between two leaves (number of branches) */

unsigned int *treepath(struct tree intree, unsigned int leaf1, unsigned int leaf2); /* sorted list of branches forming path from leaf1 to leaf2 */

int branchbetween(struct tree intree, unsigned int branch, unsigned int leaf, unsigned int otherbranch); 
/* if branch is on the path form leaf to other branch, then 1, else 0 */

struct tree treecopy(struct tree intree); /* copy tree */

struct tree addleaf(struct tree intree, unsigned int branchindex, char *newleaf); /* adding new leaf to the branch with the given index*/
struct tree nni(struct tree intree, unsigned int branchindex, unsigned int which);
/* nni number _which_ in branch _branchindex */

float globallength(struct tree intree); /* sum of lengths of all branches */

struct distancematrix readphylipmatrix(FILE *infile);
struct distancematrix readembossmatrix(FILE *infile);

unsigned int whichsplitmat(unsigned int a, unsigned int b, unsigned int c, unsigned int d, struct distancematrix matrix);
unsigned int whichsplittree(unsigned int a, unsigned int b, unsigned int c, unsigned int d, struct tree intree);

struct tree upgma(struct distancematrix inmatrix);
struct tree wpgma(struct distancematrix inmatrix, float **outweight);
struct tree wpgma2(struct distancematrix inmatrix);

int topocriterium(struct distancematrix inmatrix, struct tree intree);
float additcriterium(struct distancematrix inmatrix, struct tree intree);
float additcriterium2(struct distancematrix inmatrix, struct tree intree);
float additcriterium3(struct distancematrix inmatrix, struct tree intree);
float additcriterium4(struct distancematrix inmatrix, struct tree intree);
float bmecriterium(struct distancematrix inmatrix, struct tree intree);

int topotryaddleaf(struct distancematrix matrix, struct tree intree, char *newleaf, unsigned int branchindex);
int topotryaddleaf_b(float **distances, struct tree intree, unsigned int newleafindex, unsigned int branchindex);

struct distancematrix partmatrix(struct distancematrix inmatrix, unsigned int part);

void removetree(struct tree *totree);
void removematrix(struct distancematrix *tomatrix);

int branchdist(struct tree tree1, struct tree tree2);

int puttree_f(struct tree newtree, float newquality, 
               struct tree *curtrees, unsigned int *tokeeped, unsigned int keep, float *qualities);

int puttree_i(struct tree newtree, int newquality, 
               struct tree *curtrees, unsigned int *tokeeped, unsigned int keep, int *qualities);

struct tree subtree(struct tree intree, char ** list, unsigned int listlen);

char *writephylipmatrix(struct distancematrix inmatrix);

#endif
